###########################################################################
#  Module Name:        Low level interrupt handler for JN5148 FreeRTOS port
#
#  Created By:         Ian Morris
#
#  Original Release:   January 21, 2010
#
#  Module Description: Handles interrupts required by FreeRTOS to switch
#                      tasks, automatically using the tick timer or
#                      manually using a system call.
#
###########################################################################

###########################################################################
# Tick Timer Interrupt Handler
###########################################################################

        .type _vTickTimerHandler,@function
        .globl _vTickTimerHandler

_vTickTimerHandler:

	# Most context saving is performed by ROM based interrupt handler

        # Store critical section nesting depth on stack
        b.addi  r1, r1, 0xfffffffc
        b.addi  r3, r0, (_ulCriticalNesting)
        b.lwz   r3, 0x0(r3)
        b.sw    0x0(r1), r3

        # Save top of stack value to TCB
        b.addi  r3, r0, (_pxCurrentTCB)
        b.lwz   r3, 0x0(r3)
        b.sw    0x0(r3), r1

        # Clear tick timer int pending flag
        b.mfspr r3, r0, 0x5000        
        b.addi  r4, r0, (0xefffffff)        
        b.and   r3, r3, r4
        b.mtspr r0, r3, 0x5000

	b.jal   _vTaskIncrementTick
        b.nop   0x0

	b.jal   _vTaskSwitchContext
        b.nop   0x0

        b.j     _portRESTORE_CONTEXT

###########################################################################
# System Call (l.sys) Interrupt Handler
###########################################################################

        .type _vSystemCallHandler,@function
        .globl _vSystemCallHandler

_vSystemCallHandler:

	# Most context saving is performed by ROM based interrupt handler

        # Store critical section nesting depth on stack
        b.addi  r1, r1, 0xfffffffc
	b.addi  r3, r0, (_ulCriticalNesting)
        b.lwz   r3, 0x0(r3)
        b.sw    0x0(r1), r3

        # Save top of stack value to TCB
	b.addi  r3, r0, (_pxCurrentTCB)
        b.lwz   r3, 0x0(r3)
        b.sw    0x0(r3),r1

        b.jal   _vTaskSwitchContext
        b.nop   0x0

        b.j     _portRESTORE_CONTEXT

###########################################################################
# Restore task context
###########################################################################

        .type _portRESTORE_CONTEXT,@function
        .globl _portRESTORE_CONTEXT

_portRESTORE_CONTEXT:

        # Restore stack pointer from TCB
        b.addi  r3, r0, (_pxCurrentTCB)
        b.lwz   r3, 0x0(r3)
        b.lwz   r1, 0x0(r3)

        # Restore critical section next depth from stack
        b.addi  r3, r0, (_ulCriticalNesting)
        b.lwz   r4,0   * 4(r1)
        b.sw    0x0(r3), r4


        # Restore registers from stack
        b.lwz   r0,1   * 4(r1)
        b.lwz   r2,3   * 4(r1)
        b.lwz   r5,6   * 4(r1)
        b.lwz   r6,7   * 4(r1)
        b.lwz   r7,8   * 4(r1)
        b.lwz   r8,9   * 4(r1)
        b.lwz   r9,10  * 4(r1)
        b.lwz   r10,11 * 4(r1)
        b.lwz   r11,12 * 4(r1)
        b.lwz   r12,13 * 4(r1)
        b.lwz   r13,14 * 4(r1)
        b.lwz   r14,15 * 4(r1)
        b.lwz   r15,16 * 4(r1)

        # Don't bother restoring item at
        # stack location 17 * 4 as I have 
        # no idea where to put it - could 
        # be the IRQ number......

	# Restore ESR
	b.lwz 	 r4,18 * 4(r1)
	bw.mtspr r0, r4, 64   

	# Restore EPCR
	b.lwz 	 r4, 19 * 4(r1)
	bw.mtspr r0,  r4, 32  

	# Restore EEAR
        b.lwz 	 r4, 20 * 4(r1)
	bw.mtspr r0,  r4, 48  

        b.lwz   r3,4   * 4(r1)
        b.lwz   r4,5   * 4(r1)
        b.addi  r1,r1, 84
        bt.rfe
        b.nop	0x0

###########################################################################
# End
###########################################################################
