/*
	FreeRTOS.org V6.0.5 - Copyright (C) 2003-2008 Richard Barry.

	This file is part of the FreeRTOS.org distribution.

	FreeRTOS.org is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	FreeRTOS.org is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with FreeRTOS.org; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	A special exception to the GPL can be applied should you wish to distribute
	a combined work that includes FreeRTOS.org, without being obliged to provide
	the source code for any proprietary components.  See the licensing section
	of http://www.FreeRTOS.org for full details of how and when the exception
	can be applied.

	***************************************************************************

	Please ensure to read the configuration and relevant port sections of the
	online documentation.

	+++ http://www.FreeRTOS.org +++
	Documentation, latest information, license and contact details.

	+++ http://www.SafeRTOS.com +++
	A version that is certified for use in safety critical systems.

	+++ http://www.OpenRTOS.com +++
	Commercial support, development, porting, licensing and training services.

	***************************************************************************
*/


/* BASIC INTERRUPT DRIVEN SERIAL PORT DRIVER.
 *
 * This file only supports UART 1
 */

/* Standard includes. */
#include <stdlib.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "queue.h"
#include "task.h"

/* Demo application includes. */
#include "serial.h"

/* Platform Includes */
#include <AppHardwareApi.h>

/* Misc. constants. */
#define serNO_BLOCK				( ( portTickType ) 0 )

/* The queue used to hold received characters. */
static xQueueHandle xRxedChars;

/* The queue used to hold characters waiting transmission. */
static xQueueHandle xCharsForTx;

/* Interrupt Handler */
static void vHandleUart1Interrupt(unsigned portBASE_TYPE uxDevice, unsigned portBASE_TYPE uxItemBitmap);

/*-----------------------------------------------------------*/

xComPortHandle xSerialPortInitMinimal( unsigned portLONG ulWantedBaud, unsigned portBASE_TYPE uxQueueLength )
{
unsigned portSHORT usBaudRateDivisor;

	/* Initialise the hardware. */

	/* Generate the baud rate constants for the wanted baud rate. */
    usBaudRateDivisor = 1000000UL / ulWantedBaud;
    usBaudRateDivisor = 52; /* TODO - Hardcoded for 19200 */

	portENTER_CRITICAL();
	{
		/* Create the queues used by the com test task. */
		xRxedChars = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );
		xCharsForTx = xQueueCreate( uxQueueLength, ( unsigned portBASE_TYPE ) sizeof( signed portCHAR ) );

	    /* Enable and reset required UART */
	    vAHI_UartEnable(1);
	    vAHI_UartReset(1, pdTRUE, pdTRUE);
	    vAHI_UartReset(1, pdFALSE, pdFALSE);

	    /* Disable hardware flow control */
	    vAHI_UartSetRTSCTS(1, pdFALSE);

    	/* Register function that will handle UART interrupts */
        vAHI_Uart1RegisterCallback((PR_HWINT_APPCALLBACK)vHandleUart1Interrupt);

		/* Set baud rate, stops bits etc */
	    vAHI_UartSetBaudDivisor(1, usBaudRateDivisor);
	    vAHI_UartSetControl(1, pdFALSE, pdFALSE, E_AHI_UART_WORD_LEN_8, pdTRUE, pdFALSE); /* 8-N-1 */
	    vAHI_UartSetInterrupt(1, pdFALSE, pdFALSE, pdTRUE, pdTRUE, E_AHI_UART_FIFO_LEVEL_1);
	}
	portEXIT_CRITICAL();

	/* Unlike other ports, this serial code does not allow for more than one
	com port.  We therefore don't return a pointer to a port structure and can
	instead just return NULL. */
	return NULL;
}
/*-----------------------------------------------------------*/

signed portBASE_TYPE xSerialGetChar( xComPortHandle pxPort, signed portCHAR *pcRxedChar, portTickType xBlockTime )
{
	/* Get the next character from the buffer.  Return false if no characters
	are available, or arrive before xBlockTime expires. */
	if( xQueueReceive( xRxedChars, pcRxedChar, xBlockTime ) )
	{
		return pdTRUE;
	}
	else
	{
		return pdFALSE;
	}
}
/*-----------------------------------------------------------*/

signed portBASE_TYPE xSerialPutChar( xComPortHandle pxPort, signed portCHAR cOutChar, portTickType xBlockTime )
{
signed portBASE_TYPE xReturn = pdFAIL;

	/* Transmit a character. */
	portENTER_CRITICAL();
	{
        if (u8AHI_UartReadLineStatus(1) & E_AHI_UART_LS_THRE)
        {
			/* If THRE is true then the UART Tx ISR has indicated that
			there are no characters queued to be transmitted - so we can
			write the character directly to the shift Tx register. */
			vAHI_UartWriteData(1, cOutChar);
			xReturn = pdPASS;
        }
        else
        {
			/* THRE is false, so there are still characters waiting to be
			transmitted.  We have to queue this character so it gets
			transmitted	in turn. */

			/* Return false if after the block time there is no room on the Tx
			queue.  It is ok to block inside a critical section as each task
			maintains it's own critical section status. */
			xReturn = xQueueSend( xCharsForTx, &cOutChar, xBlockTime );

			/* Depending on queue sizing and task prioritisation:  While we
			were blocked waiting to post on the queue interrupts were not
			disabled.  It is possible that the serial ISR has emptied the
			Tx queue, in which case we need to start the Tx off again
			writing directly to the Tx register. */
			if( ( u8AHI_UartReadLineStatus(1) & E_AHI_UART_LS_THRE ) && ( xReturn == pdPASS ) )
			{
		 		/* Get back the character we just posted. */
				xQueueReceive( xCharsForTx, &cOutChar, serNO_BLOCK );
				vAHI_UartWriteData(1, cOutChar);
			}
		}
	}
	portEXIT_CRITICAL();

	return xReturn;
}
/*-----------------------------------------------------------*/

/*
 * UART 1 interrupt service routine.
 */
static void vHandleUart1Interrupt(unsigned portBASE_TYPE uxDevice, unsigned portBASE_TYPE uxItemBitmap)
{
    if (uxDevice == E_AHI_DEVICE_UART1)
    {
        /* Check for receive interrupt */
        if ((uxItemBitmap & E_AHI_UART_INT_RXDATA) != 0)
        {
			signed portCHAR cChar;
			signed portBASE_TYPE xHigherPriorityTaskWoken;

			/* Get the character from the UART and post it on the queue of Rxed
   			   characters. */
			cChar = u8AHI_UartReadData(1);

			if( xQueueSendFromISR( xRxedChars, &cChar, &xHigherPriorityTaskWoken ) )
			{
				/* If the post causes a task to wake force a context switch
				   as the woken task may have a higher priority than the task we have
		   		   interrupted. */
				taskYIELD();
			}
        }
        /* Check for transmit interrupt */
        if ((uxItemBitmap & E_AHI_UART_INT_TX) != 0)
        {
			signed portCHAR cChar;
			portBASE_TYPE xTaskWoken;

			/* The previous character has been transmitted.  See if there are any
		   	   further characters waiting transmission. */

			if( xQueueReceiveFromISR( xCharsForTx, &cChar, &xTaskWoken ) == pdTRUE )
			{
				/* There was another character queued - transmit it now. */
				vAHI_UartWriteData(1, cChar);
			}
        }
    }
}
